<?php
/**
 * Reajuste.
 *
 * @package Logic
 * @author  alessandrobartels
 */
class Reajuste {

    private $_nrorg;
    private $_nrOrgPadrao;
    private $_simulacao;
    private $_dt_alt_salario;
    private $_mes_competencia;
    private $_nrTipoAlteracao;
    private $_altera_salario;
    private $_ambienteSalFonte;
    private $_ambienteSalDestino;
    private $_nrTpModSalFonte;
    private $_nrTpModSalDestino;
    private $_vrPercentual;
    private $_vrPiso;
    private $_vrTeto;
    private $_tpArredonda;
    private $_nrDigitoArredonda;
    private $_nrLimiteArredonda;
    private $_idproporcional;   //Novas adições 04/03/2015
    private $_nrqdediastrab;
    private $_nrtipovinc;
    private $_nrvincempreg;
    private $_nrestrutsind; //Novas inclusões 19/03/2015
    //Novas inlcusões 04/03/2015]
    private $_alteracao_situacao_funcional;


    /**
     * Construtor.
     *
     * @param integer               $nrOrgPadrao     Numero da Organizacao Padrao.
     * @param integer               $nrorg           Numero da Organizacao de Trabalho.
     * @param TkDateTime            $mes_competencia Mes de competencia.
     * @param boolean               $simulacao       Flag de tipo de processo.
     * @param GpeformaltsalarEntity $formulario      Instancia de GpeformaltsalarEntity.
     *
     * @author alessandrobartels
     *         alessandrobartels - 11/07/2011 - Implementacao da Organizacao Padrao.
     *         thalesvalias - 12/12/2011 - Implementacao dos dados do formulario parametrizado como objeto.
     */
    public function __construct(
        $nrOrgPadrao, $nrorg, TkDateTime $mes_competencia, $simulacao, GpeformaltsalarEntity $formulario
    ){
        $this->_nrOrgPadrao = $nrOrgPadrao;
        $this->_nrorg = $nrorg;
        $this->_simulacao = $simulacao;
        $this->_mes_competencia = $mes_competencia;

        // @todo Alertar caso a data venha diferente de 01/mm/aaaa.
        $this->_dt_alt_salario = new TkDateTime($formulario->getDtaltsalario(), TkDateTime::FORMAT_DATE);

        /* #######  FORMULARIO ####### */
        $this->_nrTipoAlteracao = $formulario->getNrtipoaltera();
        $this->_ambienteSalFonte = $formulario->getIdambsalfonte();
        $this->_ambienteSalDestino = $formulario->getIdambsaldestino();
        $this->_nrTpModSalFonte = $formulario->getNrtpmodsalfonte();
        $this->_nrTpModSalDestino = $formulario->getNrtpmodsaldest();
        $this->_vrPercentual = $formulario->getVrpercaltera();
        $this->_vrPiso = $formulario->getVrpisosalario();
        $this->_vrTeto = $formulario->getVrtetosalario();
        $this->_tpArredonda = $formulario->getIdtparredondame();
        $this->_nrDigitoArredonda = $formulario->getNrdigarredondam();
        $this->_nrLimiteArredonda = $formulario->getNrlimarredondam();
        $this->_nrtipovinc = $formulario->getNrtipovinc();  //Nova inclusão 04/03/2015 pck
        $this->_idproporcional = $formulario->getIdproporcional();
        $this->_nrqdediastrab = $formulario->getNrqdediastrab();
        $this->_nrvincempreg = $formulario->getNrvincempreg();
        $this->_nrestrutsind = $formulario->getNrestrutsind();

        $this->_altera_salario = new GpealtesalarioEntity(null, $this->_nrorg);

        $this->_pessoa = TkDI::getContainer()->pessoa;
        //Novas inclusões 04/03/2015
        $this->_alteracao_situacao_funcional = TkDI::getContainer()->alteracao_situacao_funcional;
        $this->_faltas_intervalo = TkDI::getContainer()->faltas_intervalo;
    }


    public function calculaDiasTrabalhoMes(GpevinculomEntity $vinculo, TkDateTime $competencia )
    {
        $nrorg     = $vinculo->getNrorg();
        $nrVinculo = $vinculo->getNrvinculom();
        $modalid   = \FPA::getNrModalidadeCalculo();
        $qdeMes    = 0; //Quantidade de meses
        $comp = new TkDateTime($competencia);

        //loop para percorrer 1 ano baseado na competencia (mes final)
        for($i = 0; $i < 12; $i++){
            //Calculando a quantidade de dias trabalhados no período informado
            $dt_ini = new TkDateTime($comp); //Data inicial
            $dt_fim = new TkDateTime($comp); //Data final
            $dt_fim->addMonth(1);
            $dt_fim->addDay(-1);
            $this->_alteracao_situacao_funcional->retornaAlteracaoSituacaoFuncional($vinculo->getNrorg(), $vinculo->getNrvinculom(), $vinculo->getDtadmissaovinc());
            $situacao_funcional = TkDI::getContainer()->situacao_funcional;
            $afastamento = $this->_alteracao_situacao_funcional->retornaDiasAfastamento( $situacao_funcional, $dt_ini, $dt_fim, false );
            $diasTrabalho = 0;
            foreach($afastamento as $situacao)  //Contando a quantidade de dias trabalhados
                if($situacao == 1)              //Situação funcional normal
                    $diasTrabalho++;

            //Calculando a quantidade de faltas
            $dtComp = $dt_ini->toString('d/M/Y');
            $sql = <<<SQL
SELECT
  SUM( DECODE( EVH.IDFALTASFERIAS, 'SOMA',     ICF.VRREFFOLHA,
                                   'SUBTRAI', -ICF.VRREFFOLHA, 0 ) ) FALTAS_DIAS --*/
FROM FPA_ITECALCFOLHA ICF
JOIN FPA_CALCULOFOLHA CF ON CF.NRORG            = ICF.NRORG
                        AND CF.NRCALCULOFOLHA   = ICF.NRCALCULOFOLHA
                        AND CF.NRTIPOMOVIMENT = 1
                        AND CF.NRTPMODALIDCAL = $modalid
                        AND CF.DTOCORRENCIA = '$dtComp'
JOIN FPA_EVENTOH EVH ON ( EVH.NRORG = 0 OR EVH.NRORG = ICF.NRORG )
                    AND EVH.NREVENTOM = ICF.NREVENTOM
                    --AND EVH.IDFALTASFERIAS = 'SOMA'
                    AND EVH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM FPA_EVENTOH
                                               WHERE DTMESCOMPETENC <= '$dtComp'
                                                 AND ( NRORG = 0 OR NRORG = ICF.NRORG )
                                                 AND NREVENTOM = ICF.NREVENTOM )
--
WHERE ICF.NRORG      = $nrorg
  AND ICF.NRVINCULOM = $nrVinculo
SQL;
            $sqlResult = \Setforms2\Runtime\Framework::getInstance()->fetchAll($sql);
            $faltasDias = (int)$sqlResult[0]['FALTAS_DIAS'];

            //Verificando se a quantidade de dias trabalhados no mês atendeu o parametro do usuário
            if(($diasTrabalho - $faltasDias) >= $this->_nrqdediastrab){
                $qdeMes++;
            }

            //Atualizando a competência para o mês anterior
            $comp->addMonth(-1);
        }
        return $qdeMes;
    }


    /**
     * Executa a acao de reajuste salarial.
     *
     * @param GpevinculomEntity              $vinculo                   Instancia de GpevinculomEntity.
     * @param DemonstrativoAlteracaoSalarial $demonstrativo_alt_salario Instancia de DemonstrativoAlteracaoSalarial.
     *
     * @return DemonstrativoAlteracaoSalarial|boolean
     *
     * @author alessandrobartels
     *         alessandrobartels - 11/07/2011 - Implementacao da Organizacao Padrao.
     *         thalesvalias - 12/12/2011 - Valores parametrizados no objeto de formulario.
     */
    public function reajustar(
        GpevinculomEntity $vinculo, DemonstrativoAlteracaoSalarial $demonstrativo_alt_salario, $competencia
    ) {
        /* @todo Trazer o nome da pessoa - verificar no Calculo */
        $nrVinculo = $vinculo->getNrvinculom();

        // Define o tipo de mobilidade salarial fonte(vinculo ou operador).
        if ($this->_ambienteSalFonte === GpeFormaltsalarEntity::IDAMBSALFONTE_REAL) {
            $nrModalidade = $vinculo->historico->getNrtpmodalidsal();
        } else {
            $nrModalidade = $this->_nrTpModSalFonte;
        }

        // Carrega as alteracoes salariais.
        $carregou = $this->_altera_salario->carregaAlteracaoSalario($this->_dt_alt_salario, $nrVinculo, $nrModalidade);
        if ($carregou === true) {
            $salario_vinculo    = $this->_altera_salario->getSalario();
            $nrTipoSalario      = $this->_altera_salario->getTipoSalario();

            if($this->_idproporcional == 'N') { //Verifica se deve ou não fazer o cálculo proporcional
                $valor_reajuste = $salario_vinculo * $this->_vrPercentual / 100; //Original, cálculo comum de reajuste
            } else {
                if((int)$this->_nrqdediastrab == 0) {
                    tk_abort('Para cálculo porporcional é necessário informar a quantide de dias trabalhados no mês.');
                }
                $qdeMesesSal = $this->calculaDiasTrabalhoMes($vinculo, $competencia); //Retorna a quantidade de meses que houve dias trabalhados igual ou superior ao parametrizado via formulário
                $valor_reajuste = ( ( (int)$this->_vrPercentual / 100 ) / 12 * $qdeMesesSal ) * $salario_vinculo;  //Cálculo proporcional
            }

            $salario_reajustado = $salario_vinculo + $valor_reajuste;

            if (empty($this->_vrPiso) !== true) {
                if ($salario_reajustado < $this->_vrPiso) {
                    $salario_reajustado = $this->_vrPiso;
                }
            }

            if (empty($this->_vrTeto) !== true) {
                if ($salario_reajustado > $this->_vrTeto) {
                    $salario_reajustado = $this->_vrTeto;
                    if($salario_vinculo > $salario_reajustado) { //O novo salário do vínculo não pode ser menor que o salario atual.
                        $salario_reajustado = $salario_vinculo;
                    }
                }
            }

            $salario_reajustado = Valor::arredonda(
                $salario_reajustado, $this->_tpArredonda, $this->_nrDigitoArredonda, $this->_nrLimiteArredonda
            );

            $vr_diferenca_real = $salario_reajustado - $salario_vinculo;
            $vr_percentual_real = ($vr_diferenca_real * 100) / $salario_vinculo;

            if ($this->_simulacao === true) {
                // Monta exibicao.
                $pessoa = $this->_pessoa->retornaPessoaAtual($vinculo->getNrpessoa());
                $nmPessoa = $pessoa->historico->getNmpessoa();

                $nmVinculo = $vinculo->getNrvinculom().' - '.$nmPessoa;
                return $demonstrativo_alt_salario->addRegistros(
                    $nmVinculo, $salario_vinculo, $salario_reajustado, $vr_percentual_real, $vr_diferenca_real );
            } else {
                /* @todo - Verificar se existe alteracao com data superior a data parametrizada. Se existir o valor a
                 * ser alterado nao pode ser maior
                 */

                // Define o tipo de mobilidade salarial de destino (vinculo ou operador).
                if ($this->_ambienteSalDestino === GpeFormaltsalarEntity::IDAMBSALDESTINO_REAL) {
                    $nrModalidade = $vinculo->historico->getNrtpmodalidsal();
                } else {
                    $nrModalidade = $this->_nrTpModSalDestino;
                }

                $this->_altera_salario->geraAlteracaoSalarial( //Inversão proposital dos parâmetros vr_percentual_real e _vrPercentual
                    $nrVinculo, $salario_reajustado, $this->_dt_alt_salario, $vr_percentual_real, $this->_vrPercentual,
                    $vr_diferenca_real, $this->_nrTipoAlteracao, $nrTipoSalario, $nrModalidade
                );

                /* $this->_altera_salario->persist(); */
                return true;
            }
        } else {
            tk_warning('Nao foi encontrado registros de alteracoes salariais.');
        }

    }


    /**
     * Enter description here ...
     *
     * @param GpevinculomEntity $vinculo Instancia de GpevinculomEntity.
     *
     * @return void
     */
    public function retrocesso(GpevinculomEntity $vinculo){

        $nrVinculo = $vinculo->getNrvinculom();

        if ($this->_ambienteSalFonte === GpeFormaltsalarEntity::IDAMBSALFONTE_REAL) {
            $nrModalidade = $vinculo->historico->getNrtpmodalidsal();
        } else {
            $nrModalidade = $this->_nrTpModSalDestino;
        }

        $itens = $this->_altera_salario->carregaListaAlteracaoSalarioData($this->_dt_alt_salario, $nrVinculo);

        $removeu = false;
        foreach ($itens as $item) {
            if (($item->getNrtipoaltera() === $this->_nrTipoAlteracao)
                && ($item->getNrtpmodalidsal() === $nrModalidade)
            ) {
                $this->_altera_salario->removeItem($item);
                $removeu = true;
            }
        }

        if ($removeu === true) {
            $this->_altera_salario->persist();
            tk_warning('Retrocesso - OK');
        } else {
            tk_warning('Nenhuma alteracao encontrada.');
        }

    }


}